/*
** USB_SELF_LOOP.c
**
** Copyright  2011 Future Devices International Limited
**
** C Source file for Vinculum II test applications
** Main module
**
** Author: FTDI
** Project: Vinculum II
** Module: Vinculum II Applications
** Requires: VOS
** Comments:
**
**This is a project designed to show how to use VNC2 USB Host port with
**FT232 class host driver.
**
**
**VNC2 USB port 2 is the host
**
**
**After studying this project the developer should be able to:
**Open the USB host port.
**Read / Write data with the USB Host
**Read/Write data with the UART port.
**
**
**ALL CODE SUPPLIED HERE IS PROVIDED ON AN AS IS BASIS.
**NO GUARANTEE OF FUNCTION IS PROVIDED.
**
**
** History:
**  1  Initial version
**
*********************************************************************/

#include "vos.h"

/* FTDI:SHF Header Files */
#include "USB.h"
#include "USBHost.h"
#include "ioctl.h"
#include "USBHostFT232.h"
#include "UART.h"
#include "stdio.h"
#include "string.h"
#include "errno.h"
#include "stdlib.h"
/* FTDI:EHF */


/*FTDI:control blocks*/

//FT232 common control block
common_ioctl_cb_t CommonIoctl;

//FT232Host attach control block
usbhostft232_ioctl_cb_attach_t ft232HostAttach;

//USB Host control block
usbhost_ioctl_cb_t hc_iocb;

//USB Host class Control block
usbhost_ioctl_cb_class_t hc_iocb_class;

//USB Host vendor Control block
usbhost_ioctl_cb_vid_pid_t hc_iocb_vendor;




/* End FTDI:control blocks*/



/* FTDI:SDC Driver Constants */
#define VOS_DEV_USBHOST_2 0
#define VOS_DEV_USBHOST_FT232 1
#define VOS_DEV_UART 2

#define VOS_NUMBER_DEVICES 3
/* FTDI:EDC */

/* FTDI:SDH Driver Handles */
VOS_HANDLE hUSBHOST_2; // USB Host Port 2
VOS_HANDLE hUSBHOST_FT232; // Connects to FT232 devices using the USB Host Interface
VOS_HANDLE hUART; //handle for UART port

//handle for device connected to USB host
usbhost_device_handle *ifDev2;
/* FTDI:EDH */

/* FTDI:STP Thread Prototypes */
vos_tcb_t *tcbFIRMWARE;

void firmware();
/* FTDI:ETP */

/* Declaration for IOMUx setup function */
void iomux_setup(void);

/* initialise devices */
void init_devices(void)
{
	// UART Driver configuration context
	uart_context_t uartContext;


	// USB Host configuration context
	usbhost_context_t usbhostContext;


	// Initialise USB Host
	usbhostContext.if_count = 8;
	usbhostContext.ep_count = 16;
	usbhostContext.xfer_count = 2;
	usbhostContext.iso_xfer_count = 2;
	usbhost_init(-1, VOS_DEV_USBHOST_2, &usbhostContext);

	// Initialise USB Host FT232 Driver
	usbHostFt232_init(VOS_DEV_USBHOST_FT232);




	// Initialise UART
	uartContext.buffer_size = VOS_BUFFER_SIZE_128_BYTES;
	uart_init(VOS_DEV_UART,&uartContext);



}

/***********************************************************************
/* Main code - entry point to firmware */
/**********************************************************************/

void main(void)
{


	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(512);
	/* FTDI:EKI */

	iomux_setup();

	init_devices();

	/* FTDI:SCT Thread Creation */
	tcbFIRMWARE = vos_create_thread_ex(20, 4096, firmware, "Application", 0);
	/* FTDI:ECT */

	vos_start_scheduler();

main_loop:
	goto main_loop;
}

/***********************************************************************
/* Application Threads */
/**********************************************************************/
void firmware()
{

unsigned char i;
enum USBHOST_STATUS usbStatus;
unsigned char status;

unsigned char buf_USB_to_UART[64];
unsigned char buf_UART_to_USB[64];


unsigned short counter;
unsigned short num_read;
unsigned short num_written;



//open UART port
hUART = vos_dev_open(VOS_DEV_UART);


//configure UART port

		//set uart baud rate to 115200
        CommonIoctl.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
        CommonIoctl.set.uart_baud_rate = UART_BAUD_115200;
        //send setup command to uart(determined by handle)
        vos_dev_ioctl(hUART, &CommonIoctl);

        //set uart data bits to 8
        CommonIoctl.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
        CommonIoctl.set.param = UART_DATA_BITS_8;
        //send setup command to uart(determined by handle)
        vos_dev_ioctl(hUART, &CommonIoctl);

        //set uart stop bits to 2
        CommonIoctl.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
        CommonIoctl.set.param = UART_STOP_BITS_1;
        //send setup command to uart(determined by handle)
        vos_dev_ioctl(hUART, &CommonIoctl);

        //set uart parity to none
        CommonIoctl.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
        CommonIoctl.set.param = UART_PARITY_NONE;
        //send setup command to uart(determined by handle)
        vos_dev_ioctl(hUART, &CommonIoctl);

        //set uart flow control to none
        CommonIoctl.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
        CommonIoctl.set.param = UART_FLOW_RTS_CTS;
        //send setup command to uart(determined by handle)
        vos_dev_ioctl(hUART, &CommonIoctl);

		//enable UART DMA
		CommonIoctl.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
		CommonIoctl.set.param = DMA_ACQUIRE_AND_RETAIN;
		vos_dev_ioctl(hUART,&CommonIoctl);





//open host driver and FT232 host class driver
hUSBHOST_2 = vos_dev_open(VOS_DEV_USBHOST_2);

/******************************************************************
//Loop to check device is enumerated and send data back and forward
/*****************************************************************/


do
{
		//wait for enumeration to complete
		vos_delay_msecs(250);
		// user ioctl to see if bus available
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_GET_CONNECT_STATE;
		hc_iocb.get = &i;
		vos_dev_ioctl(hUSBHOST_2, &hc_iocb); //checking USB port2 - the host

		if (i == PORT_STATE_ENUMERATED)
		{
			status = 0;

			// find FTDI vendor device
			hc_iocb_vendor.vid = USB_VID_FTDI;
			hc_iocb_vendor.pid = USB_PID_ANY;

			// user ioctl to find first hub device
			hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_FIND_HANDLE_BY_VID_PID;
			hc_iocb.handle.dif = NULL;
			hc_iocb.set = &hc_iocb_vendor;
			hc_iocb.get = &ifDev2;

			usbStatus = vos_dev_ioctl(hUSBHOST_2, &hc_iocb);
			if (usbStatus != USBHOST_OK)
			{
				status = 1;
			}

			if (status == 0)
			{
				// now we have a device, intialise a driver for it
				hUSBHOST_FT232 = vos_dev_open(VOS_DEV_USBHOST_FT232);

				// ft232_attach
				ft232HostAttach.hc_handle = hUSBHOST_2;
				ft232HostAttach.ifDev = ifDev2;
				ft232HostAttach.ftPort = USBHOSTFT232_PORTA;

				CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_ATTACH;
				CommonIoctl.set = &ft232HostAttach;

				if (vos_dev_ioctl(hUSBHOST_FT232, &CommonIoctl) != USBHOSTFT232_OK)
				{
						status = 1;
				}
				else
				{

/******************************************************************
//Now that device is connected it must be configured
//NOTE BAUD RATE uses set.uart_baud_rate, all other parameters are
//set with set.param.
/*****************************************************************/

					// Set baud rate 9600 baud
					CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_BAUD_RATE;
					CommonIoctl.set.uart_baud_rate = USBHOSTFT232_BAUD_115200;
					vos_dev_ioctl(hUSBHOST_FT232,&CommonIoctl);

					// Set number of data bits to 8
					CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_DATA_BITS;
					CommonIoctl.set.param = USBHOSTFT232_DATA_BITS_8;
					vos_dev_ioctl(hUSBHOST_FT232,&CommonIoctl);

					// Set number of stop bits to 1
					CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_STOP_BITS;
					CommonIoctl.set.param = USBHOSTFT232_STOP_BITS_1;
					vos_dev_ioctl(hUSBHOST_FT232,&CommonIoctl);

					// Set parity to none
					CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_PARITY;
					CommonIoctl.set.param = USBHOSTFT232_PARITY_NONE;
					vos_dev_ioctl(hUSBHOST_FT232,&CommonIoctl);

					// Set flow control to none
					CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_SET_FLOW_CONTROL;
					CommonIoctl.set.param = USBHOSTFT232_FLOW_NONE;
					vos_dev_ioctl(hUSBHOST_FT232,&CommonIoctl);

					//start polling the connected device
					CommonIoctl.ioctl_code = VOS_IOCTL_USBHOSTFT232_START_POLL;
					vos_dev_ioctl(hUSBHOST_FT232,&CommonIoctl);




/******************************************************************
//Loop data between UART and UAB port
/*****************************************************************/


			//UART checking queue status

			memset(buf_UART_to_USB,0x0,64);

 			num_written = 0;
 			CommonIoctl.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;

		 	vos_dev_ioctl(hUART, &CommonIoctl);

 			num_written = CommonIoctl.get.queue_stat;

			if (num_written != 0){

			//UART reading
 			num_read = 0;
			if (vos_dev_read(hUART, buf_UART_to_USB, num_written, &num_read) == UART_OK)

 					  {
                        // write it out the USB port.
						vos_delay_msecs(5); //wait for data to go

						counter = 0;
						vos_dev_write(hUSBHOST_FT232, buf_UART_to_USB, num_read, &counter);


						vos_delay_msecs(5); //wait for data to go


                      }
				}

			//USB host checking for data

			num_written = 0;
 			CommonIoctl.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;

		 	vos_dev_ioctl(hUSBHOST_FT232, &CommonIoctl);

 			num_written = CommonIoctl.get.queue_stat;

			if (num_written != 0){
			//USB HOST READING
			memset(buf_USB_to_UART,0x0,64);

			num_read = 0;
   	if (vos_dev_read(hUSBHOST_FT232, buf_USB_to_UART, num_written, &num_read) == USBHOSTFT232_OK)

                      {
                        // write it out the UART port

						vos_delay_msecs(5);

						counter = 0;
						vos_dev_write(hUART, buf_USB_to_UART, num_read, &counter);

						vos_delay_msecs(5); //wait for data to go

                      }
				}
		}
			}

		}
	} while (1);






}

